var mediaObjects = {};

var AlternativeMedia = {};

/**
 * This class provides access to the device media, interfaces to both sound and video
 *
 * @constructor
 * @param src                   The file name or url to play
 * @param successCallback       The callback to be called when the file is done playing or recording.
 *                                  successCallback()
 * @param errorCallback         The callback to be called if there is an error.
 *                                  errorCallback(int errorCode) - OPTIONAL
 * @param statusCallback        The callback to be called when media status has changed.
 *                                  statusCallback(int statusCode) - OPTIONAL
 */
var WebAppMedia = function(src, successCallback, errorCallback, statusCallback) {
    this.id = (new Date()).getTime();
    mediaObjects[this.id] = this;
    this.src = src;
    this.audio = new Audio(src);
    this.successCallback = successCallback;
    this.errorCallback = errorCallback;
    this.statusCallback = statusCallback;
    this._duration = -1;
    this._position = -1;

    var me = this;

    this.audio.onplay = function() {
        AlternativeMedia.onStatus(me.id, AlternativeMedia.MEDIA_STATE, AlternativeMedia.MEDIA_RUNNING);
    };

    this.audio.onpause = function() {
        AlternativeMedia.onStatus(me.id, AlternativeMedia.MEDIA_STATE, AlternativeMedia.MEDIA_PAUSED);
    };
};

// AlternativeMedia messages
AlternativeMedia.MEDIA_STATE = 1;
AlternativeMedia.MEDIA_DURATION = 2;
AlternativeMedia.MEDIA_POSITION = 3;
AlternativeMedia.MEDIA_ERROR = 9;

// AlternativeMedia states
AlternativeMedia.MEDIA_NONE = 0;
AlternativeMedia.MEDIA_STARTING = 1;
AlternativeMedia.MEDIA_RUNNING = 2;
AlternativeMedia.MEDIA_PAUSED = 3;
AlternativeMedia.MEDIA_STOPPED = 4;
AlternativeMedia.MEDIA_MSG = ["None", "Starting", "Running", "Paused", "Stopped"];

// "static" function to return existing objs.
AlternativeMedia.get = function(id) {
    return mediaObjects[id];
};

/**
 * Start or resume playing audio file.
 */
WebAppMedia.prototype.play = function(options) {
    this.audio.play();
};

/**
 * Stop playing audio file.
 */
WebAppMedia.prototype.stop = function() {
    this.audio.pause();
    this.audio.currentTime = 0;
    this._position = 0;
};

/**
 * Seek or jump to a new time in the track..
 */
WebAppMedia.prototype.seekTo = function(milliseconds) {
    this._position = milliseconds / 1000;
    this.audio.currentTime = milliseconds / 1000;
};

/**
 * Pause playing audio file.
 */
WebAppMedia.prototype.pause = function() {
    this.audio.pause();
};

/**
 * Get duration of an audio file.
 * The duration is only set for audio that is playing, paused or stopped.
 *
 * @return      duration or -1 if not known.
 */
WebAppMedia.prototype.getDuration = function() {
    return this.audio.duration;
};

/**
 * Get position of audio.
 */
WebAppMedia.prototype.getCurrentPosition = function(success, fail) {
    this._position = this.audio.currentTime;
    success(this.audio.currentTime);
};

/**
 * Release the resources.
 */
WebAppMedia.prototype.release = function() {
    this.audio.pause();
};

/**
 * Adjust the volume.
 */
WebAppMedia.prototype.setVolume = function(volume) {
    console.log('setvolume');
};

/**
 * Adjust the playback rate.
 */
WebAppMedia.prototype.setRate = function(rate) {
    console.log('setRate');
};

/**
 * Get amplitude of audio.
 */
WebAppMedia.prototype.getCurrentAmplitude = function(success, fail) {
    console.log('getCurrentAmplitude');
};

/**
 * Audio has status update.
 * PRIVATE
 *
 * @param id            The media object id (string)
 * @param msgType       The 'type' of update this is
 * @param value         Use of value is determined by the msgType
 */
AlternativeMedia.onStatus = function(id, msgType, value) {
    var media = mediaObjects[id];

    if (media) {
        switch(msgType) {
            case AlternativeMedia.MEDIA_STATE :
                if (media.statusCallback) {
                    media.statusCallback(value);
                }
                if (value == AlternativeMedia.MEDIA_STOPPED) {
                    if (media.successCallback) {
                        media.successCallback();
                    }
                }
                break;
            case AlternativeMedia.MEDIA_DURATION :
                media._duration = value;
                break;
            case AlternativeMedia.MEDIA_ERROR :
                if (media.errorCallback) {
                    media.errorCallback(value);
                }
                break;
            case AlternativeMedia.MEDIA_POSITION :
                media._position = Number(value);
                break;
            default :
                if (console.error) {
                    console.error("Unhandled AlternativeMedia.onStatus :: " + msgType);
                }
                break;
        }
    } else if (console.error) {
        console.error("Received AlternativeMedia.onStatus callback for unknown media :: " + id);
    }

};
